﻿Public Class frmRmTableEdit
	Public Sub New()
		InitializeComponent()
		AutoMenu.Create(AddressOf ReadInputText, AddressOf WriteOutputText, AddressOf ColumnDupl, AddressOf RowDupl, AddressOf ClearSelection, AddressOf ColDelete)
		ReadInputText()
	End Sub

	Private _InputEntries As List(Of List(Of String))

	Private Sub ColumnDupl()
		Dim pos = CurrCellPos()
		If pos.IsEmpty Then Return
		_InputEntries = GetOutputEntries()
		Dim i0 = _InputEntries.FindLastIndex(pos.Y, Function(x) x.Count <= pos.X)
		Dim i1 = _InputEntries.FindIndex(pos.Y, Function(x) x.Count <= pos.X)
		For i = i0 + 1 To i1 - 1
			Dim rw = _InputEntries(i)
			rw.Insert(pos.X, rw(pos.X))
		Next
		RebuildGrid(pos)
	End Sub
	Private Sub RowDupl()
		Dim pos = CurrCellPos()
		If pos.IsEmpty Then Return
		_InputEntries = GetOutputEntries()
		_InputEntries.Insert(pos.Y, _InputEntries(pos.Y).ToList)
		RebuildGrid(pos)
	End Sub
	Private Sub ColDelete()
		Dim pos = CurrCellPos()
		If pos.IsEmpty Then Return
		_InputEntries = GetOutputEntries()
		Dim i0 = _InputEntries.FindLastIndex(pos.Y, Function(x) x.Count <= pos.X)
		Dim i1 = _InputEntries.FindIndex(pos.Y, Function(x) x.Count <= pos.X)
		If i1 < 0 Then i1 = _InputEntries.Count
		For i = i0 + 1 To i1 - 1
			_InputEntries(i).RemoveAt(pos.X)
		Next
		RebuildGrid(pos)
	End Sub
	Private Sub ClearSelection()
		For Each cll As DataGridViewCell In Grid.SelectedCells
			cll.Value = Nothing
		Next
	End Sub

	Private Sub ReadInputText()
		_InputEntries = txtInput.Lines.ToList(Function(x) x.Split("|"c).ToList)
		If _InputEntries.Count = 0 Then Return
		For i = _InputEntries.Count - 2 To 0 Step -1
			Dim curr = _InputEntries(i)
			While curr.First = "" AndAlso curr.Last <> "" 'dann Folgezeile hochziehen
				Dim succ = _InputEntries(i + 1)
				curr.Poke($"{curr.Peek}{Lf}{succ.First}")
				curr.AddRange(succ.Skip(1))
				_InputEntries.RemoveAt(i + 1)
			End While
		Next
		RebuildGrid(Point.Empty)
		Dim entries = GetOutputEntries()
		Dim ii = Enumerable.Range(0, entries.Count).ToList.FindIndex(Function(x) Not _InputEntries(x).SequenceEqual(entries(x)))
		If ii >= 0 Then Msg("Unstimmigkeit in Zeile ", ii)
	End Sub

	Private Sub RebuildGrid(currPos As Point)
		Grid.Columns.Clear()
		Grid.Rows.Clear()
		Grid.ColumnCount = _InputEntries.Max(Function(x) x.Count)
		For Each entryRow In _InputEntries
			Grid.Rows.Add(entryRow.ToArray)
		Next
		If currPos.IsEmpty Then Return
		currPos.X.Minimize(Grid.ColumnCount - 2)
		Grid.CurrentCell = Grid(currPos.X, currPos.Y)
	End Sub

	Private Function CurrCellPos() As Point
		Dim cell = Grid.CurrentCell
		If cell.Null Then Return Point.Empty
		Dim rw = _InputEntries(cell.RowIndex)
		If Not cell.ColumnIndex.IsInRange(1, rw.Count - 2) Then Return Point.Empty
		Return New Point(cell.ColumnIndex, cell.RowIndex)
	End Function

	Private Function GetOutputEntries() As List(Of List(Of String))
		Dim entries = Grid.RowsX.ToList(Function(x) x.Cells.Cast(Of DataGridViewCell).ToList(Function(xx) Convert.ToString(xx.Value)))
		For i = 0 To entries.Count - 1
			entries(i).RemoveRangeX(_InputEntries(i).Count)
		Next
		Return entries
	End Function

	Private Sub WriteOutputText()
		Dim lines = GetOutputEntries.ToArray(Function(x) "|".Between(x))
		txtOutput.Lines = lines
	End Sub

	Private _DragOffset As Size, _ResizeCell As DataGridViewCell

	Private Sub Grid_CellMouseDown(sender As Object, e As DataGridViewCellMouseEventArgs) Handles Grid.CellMouseDown
		If e.Button <> MouseButtons.Right Then Return
		AddHandler Grid.CellMouseMove, AddressOf Grid_CellMouseMove
		_ResizeCell = Grid(e.ColumnIndex, e.RowIndex)
		Dim rct = Grid.RectangleToScreen(Grid.GetCellDisplayRectangle(_ResizeCell.ColumnIndex, _ResizeCell.RowIndex, False))
		Dim cellBottomRight = New Size(rct.Location + rct.Size)
		Dim mousePos = New Size(Control.MousePosition)
		_DragOffset = cellBottomRight - mousePos
	End Sub

	Private Sub Grid_CellMouseMove(sender As Object, e As DataGridViewCellMouseEventArgs)
		If _ResizeCell Is Nothing Then Return
		Dim rct = Grid.RectangleToScreen(Grid.GetCellDisplayRectangle(_ResizeCell.ColumnIndex, _ResizeCell.RowIndex, False))
		Dim cellBottomRight = New Size(rct.Location + rct.Size)
		Dim diff = Control.MousePosition - (cellBottomRight - _DragOffset)
		Grid.Columns(_ResizeCell.ColumnIndex).Width += diff.X
		Grid.Rows(_ResizeCell.RowIndex).Height += diff.Y
	End Sub

	Private Sub Grid_CellMouseUp(sender As Object, e As DataGridViewCellMouseEventArgs) Handles Grid.CellMouseUp
		If e.Button <> MouseButtons.Right Then Return
		RemoveHandler Grid.CellMouseMove, AddressOf Grid_CellMouseMove
		_ResizeCell = Nothing
	End Sub

End Class
